@@ -1,7 +1,7 @@
 # src/codegraphcontext/tools/code_finder.py
 import logging
 import re
-from typing import Any, Dict, List
+from typing import Any, Dict, List, Literal
 from pathlib import Path
 
 from ..core.database import DatabaseManager
@@ -15,33 +15,53 @@ class CodeFinder:
         self.db_manager = db_manager
         self.driver = self.db_manager.get_driver()
 
-    def find_by_function_name(self, search_term: str) -> List[Dict]:
-        """Find functions by name matching using the full-text index."""
-        with self.driver.session() as session:
-            result = session.run("""
-                CALL db.index.fulltext.queryNodes("code_search_index", $search_term) YIELD node, score
+    def format_query(self, find_by: Literal["Class", "Function"], fuzzy_search:bool) -> str:
+        """Format the search query based on the search type and fuzzy search settings."""
+        return f"""
+            CALL db.index.fulltext.queryNodes("code_search_index", $search_term) YIELD node, score
                 WITH node, score
-                WHERE node:Function AND node.name CONTAINS $search_term
+                WHERE node:{find_by} {'AND node.name CONTAINS $search_term' if not fuzzy_search else ''}
                 RETURN node.name as name, node.file_path as file_path, node.line_number as line_number,
-                       node.source as source, node.docstring as docstring, node.is_dependency as is_dependency
+                    node.source as source, node.docstring as docstring, node.is_dependency as is_dependency
                 ORDER BY score DESC
                 LIMIT 20
-            """, search_term=search_term)
-            return [dict(record) for record in result]
-    
-    def find_by_class_name(self, search_term: str) -> List[Dict]:
-        """Find classes by name matching using the full-text index."""
+            """
+
+    def find_by_function_name(self, search_term: str, fuzzy_search: bool) -> List[Dict]:
+        """Find functions by name matching."""
         with self.driver.session() as session:
-            result = session.run("""
-                CALL db.index.fulltext.queryNodes("code_search_index", $search_term) YIELD node, score
-                WITH node, score
-                WHERE node:Class AND node.name CONTAINS $search_term
-                RETURN node.name as name, node.file_path as file_path, node.line_number as line_number,
-                       node.source as source, node.docstring as docstring, node.is_dependency as is_dependency
-                ORDER BY score DESC
-                LIMIT 20
-            """, search_term=search_term)
-            return [dict(record) for record in result]
+            if not fuzzy_search:
+                # Use simple match for exact search to avoid fulltext index dependency
+                result = session.run("""
+                    MATCH (node:Function {name: $name})
+                    RETURN node.name as name, node.file_path as file_path, node.line_number as line_number,
+                           node.source as source, node.docstring as docstring, node.is_dependency as is_dependency
+                    LIMIT 20
+                """, name=search_term)
+                return result.data()
+            
+            # Fuzzy search using fulltext index
+            formatted_search_term = f"name:{search_term}"
+            result = session.run(self.format_query("Function", fuzzy_search), search_term=formatted_search_term)
+            return result.data()
+
+    def find_by_class_name(self, search_term: str, fuzzy_search: bool) -> List[Dict]:
+        """Find classes by name matching."""
+        with self.driver.session() as session:
+            if not fuzzy_search:
+                # Use simple match for exact search to avoid fulltext index dependency
+                result = session.run("""
+                    MATCH (node:Class {name: $name})
+                    RETURN node.name as name, node.file_path as file_path, node.line_number as line_number,
+                           node.source as source, node.docstring as docstring, node.is_dependency as is_dependency
+                    LIMIT 20
+                """, name=search_term)
+                return result.data()
+
+            # Fuzzy search using fulltext index
+            formatted_search_term = f"name:{search_term}"
+            result = session.run(self.format_query("Class", fuzzy_search), search_term=formatted_search_term)
+            return result.data()
 
     def find_by_variable_name(self, search_term: str) -> List[Dict]:
         """Find variables by name matching"""
@@ -55,8 +75,8 @@ class CodeFinder:
                 LIMIT 20
             """, search_term=search_term, regex_pattern=f"(?i).*{re.escape(search_term)}.*")
             
-            return [dict(record) for record in result]
-    
+            return result.data()
+
     def find_by_content(self, search_term: str) -> List[Dict]:
         """Find code by content matching in source or docstrings using the full-text index."""
         with self.driver.session() as session:
@@ -76,16 +96,21 @@ class CodeFinder:
                 ORDER BY score DESC
                 LIMIT 20
             """, search_term=search_term)
-            return [dict(record) for record in result]
-    
-    def find_related_code(self, user_query: str) -> Dict[str, Any]:
+            return result.data()
+
+    def find_related_code(self, user_query: str, fuzzy_search: bool, edit_distance: int) -> Dict[str, Any]:
         """Find code related to a query using multiple search strategies"""
+        if fuzzy_search:
+            user_query_normalized = " ".join(map(lambda x: f"{x}~{edit_distance}", user_query.split(" ")))
+        else:
+            user_query_normalized = user_query
+
         results = {
-            "query": user_query,
-            "functions_by_name": self.find_by_function_name(user_query),
-            "classes_by_name": self.find_by_class_name(user_query),
-            "variables_by_name": self.find_by_variable_name(user_query),
-            "content_matches": self.find_by_content(user_query)
+            "query": user_query_normalized,
+            "functions_by_name": self.find_by_function_name(user_query_normalized, fuzzy_search),
+            "classes_by_name": self.find_by_class_name(user_query_normalized, fuzzy_search),
+            "variables_by_name": self.find_by_variable_name(user_query),  # no fuzzy for variables as they are not using full-text index
+            "content_matches": self.find_by_content(user_query_normalized)
         }
         
         all_results = []
@@ -140,7 +165,7 @@ class CodeFinder:
                     LIMIT 20
                 """
                 result = session.run(query, argument_name=argument_name)
-            return [dict(record) for record in result]
+            return result.data()
 
     def find_functions_by_decorator(self, decorator_name: str, file_path: str = None) -> List[Dict]:
         """Find functions that have a specific decorator applied to them."""
@@ -165,7 +190,7 @@ class CodeFinder:
                     LIMIT 20
                 """
                 result = session.run(query, decorator_name=decorator_name)
-            return [dict(record) for record in result]
+            return result.data()
     
     def who_calls_function(self, function_name: str, file_path: str = None) -> List[Dict]:
         """Find what functions call a specific function using CALLS relationships with improved matching"""
@@ -275,7 +300,7 @@ class CodeFinder:
                     LIMIT 20
                 """, function_name=function_name)
             
-            return [dict(record) for record in result]
+            return result.data()
     
     def who_imports_module(self, module_name: str) -> List[Dict]:
         """Find what files import a specific module using IMPORTS relationships"""
@@ -300,7 +325,7 @@ class CodeFinder:
                 LIMIT 20
             """, module_name=module_name)
             
-            return [dict(record) for record in result]
+            return result.data()
     
     def who_modifies_variable(self, variable_name: str) -> List[Dict]:
         """Find what functions contain or modify a specific variable"""
@@ -331,7 +356,7 @@ class CodeFinder:
                 LIMIT 20
             """, variable_name=variable_name)
             
-            return [dict(record) for record in result]
+            return result.data()
     
     def find_class_hierarchy(self, class_name: str, file_path: str = None) -> Dict[str, Any]:
         """Find class inheritance relationships using INHERITS relationships"""
@@ -409,7 +434,7 @@ class CodeFinder:
                 LIMIT 20
             """, function_name=function_name)
             
-            return [dict(record) for record in result]
+            return result.data()
     
     def find_dead_code(self, exclude_decorated_with: List[str] = None) -> Dict[str, Any]:
         """Find potentially unused functions (not called by other functions in the project), optionally excluding those with specific decorators."""
@@ -467,7 +492,7 @@ class CodeFinder:
                     LIMIT 50
                 """
                 result = session.run(query, function_name=function_name)
-            return [dict(record) for record in result]
+            return result.data()
 
     def find_all_callees(self, function_name: str, file_path: str = None) -> List[Dict]:
         """Find all direct and indirect callees of a specific function."""
@@ -490,15 +515,16 @@ class CodeFinder:
                     LIMIT 50
                 """
                 result = session.run(query, function_name=function_name)
-            return [dict(record) for record in result]
+            return result.data()
 
     def find_function_call_chain(self, start_function: str, end_function: str, max_depth: int = 5) -> List[Dict]:
         """Find call chains between two functions"""
         with self.driver.session() as session:
+            # FalkorDB requires shortestPath in WITH/RETURN clause
             result = session.run(f"""
-                MATCH path = shortestPath(
-                    (start:Function {{name: $start_function}})-[:CALLS*1..{max_depth}]->(end:Function {{name: $end_function}})
-                )
+                MATCH (start:Function {{name: $start_function}}), (end:Function {{name: $end_function}})
+                WITH shortestPath((start)-[:CALLS*1..{max_depth}]->(end)) as path
+                WHERE path IS NOT NULL
                 WITH path, nodes(path) as func_nodes, relationships(path) as call_rels
                 RETURN 
                     [node in func_nodes | {{
@@ -517,7 +543,47 @@ class CodeFinder:
                 LIMIT 10
             """, start_function=start_function, end_function=end_function)
             
-            return [dict(record) for record in result]
+            return result.data()
+
+    def find_by_type(self, element_type: str, limit: int = 50) -> List[Dict]:
+        """Find all elements of a specific type (Function, Class, File, Module)."""
+        # Map input type to node label
+        type_map = {
+            "function": "Function",
+            "class": "Class",
+            "file": "File",
+            "module": "Module"
+        }
+        label = type_map.get(element_type.lower())
+        
+        if not label:
+            return []
+            
+        with self.driver.session() as session:
+            if label == "File":
+                query = f"""
+                    MATCH (n:File)
+                    RETURN n.name as name, n.path as file_path, n.is_dependency as is_dependency
+                    ORDER BY n.path
+                    LIMIT $limit
+                """
+            elif label == "Module":
+                query = f"""
+                    MATCH (n:Module)
+                    RETURN n.name as name, n.name as file_path, false as is_dependency
+                    ORDER BY n.name
+                    LIMIT $limit
+                """
+            else:
+                query = f"""
+                    MATCH (n:{label})
+                    RETURN n.name as name, n.file_path as file_path, n.line_number as line_number, n.is_dependency as is_dependency
+                    ORDER BY n.is_dependency ASC, n.name
+                    LIMIT $limit
+                """
+            
+            result = session.run(query, limit=limit)
+            return result.data()
     
     def find_module_dependencies(self, module_name: str) -> Dict[str, Any]:
         """Find all dependencies and dependents of a module"""
@@ -740,7 +806,7 @@ class CodeFinder:
                 """
                 result = session.run(query, function_name=function_name)
             
-            return [dict(record) for record in result]
+            return result.data()
 
     def find_most_complex_functions(self, limit: int = 10) -> List[Dict]:
         """Find the most complex functions based on cyclomatic complexity."""
@@ -753,7 +819,7 @@ class CodeFinder:
                 LIMIT $limit
             """
             result = session.run(query, limit=limit)
-            return [dict(record) for record in result]
+            return result.data()
 
     def list_indexed_repositories(self) -> List[Dict]:
         """List all indexed repositories."""
@@ -763,4 +829,4 @@ class CodeFinder:
                 RETURN r.name as name, r.path as path, r.is_dependency as is_dependency
                 ORDER BY r.name
             """)
-            return [dict(record) for record in result]
+            return result.data()
